/*
    *
    * This file is a part of CorePaint.
    * A paint app for C Suite.
    * Copyright 2019 CuboCore Group
    *

    *
    * This file is taken from EasyPaint https://github.com/Gr1N/EasyPaint
    * Suitable modifications have been made to meet the needs of CorePaint.
    *

    *
    * This program is free software; you can redistribute it and/or modify
    * it under the terms of the GNU General Public License as published by
    * the Free Software Foundation; either version 3 of the License, or
    * (at your option) any later version.
    *

    *
    * This program is distributed in the hope that it will be useful,
    * but WITHOUT ANY WARRANTY; without even the implied warranty of
    * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    * GNU General Public License for more details.
    *

    *
    * You should have received a copy of the GNU General Public License
    * along with this program; if not, write to the Free Software
    * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    * MA 02110-1301, USA.
    *
*/


#include "fillinstrument.h"


FillInstrument::FillInstrument(QObject *parent) : AbstractInstrument(parent)
{
}

void FillInstrument::mousePressEvent(QMouseEvent *event, ImageArea &imageArea)
{
    if (event->button() == Qt::LeftButton || event->button() == Qt::RightButton) {
        mStartPoint = mEndPoint = event->pos();
        imageArea.setIsPaint(true);
        makeUndoCommand(imageArea);
    }
}

void FillInstrument::mouseMoveEvent(QMouseEvent *, ImageArea &)
{

}

void FillInstrument::mouseReleaseEvent(QMouseEvent *event, ImageArea &imageArea)
{
    if (imageArea.isPaint()) {
        if (event->button() == Qt::LeftButton) {
            paint(imageArea, false);
        } else if (event->button() == Qt::RightButton) {
            paint(imageArea, true);
        }

        imageArea.setIsPaint(false);
    }
}

void FillInstrument::paint(ImageArea &imageArea, bool isSecondaryColor, bool)
{
    QColor switchColor;

    if (!isSecondaryColor) {
        switchColor = DataSingleton::Instance()->getPrimaryColor();
    } else {
        switchColor = DataSingleton::Instance()->getSecondaryColor();
    }

    QRgb pixel(imageArea.getImage()->pixel(mStartPoint));
    QColor oldColor(pixel);

    if (switchColor != oldColor) {
        fillRecurs(mStartPoint.x(), mStartPoint.y(),
                   switchColor.rgb(), oldColor.rgb(),
                   *imageArea.getImage());
    }

    imageArea.setEdited(true);
    imageArea.update();
}

void FillInstrument::fillRecurs(int x, int y, QRgb switchColor, QRgb oldColor, QImage &tempImage)
{
    int temp_x(x), left_x(0);

    while (true) {
        if (tempImage.pixel(temp_x, y) != oldColor) {
            break;
        }

        tempImage.setPixel(temp_x, y, switchColor);

        if (temp_x > 0) {
            --temp_x;
            left_x = temp_x;
        } else {
            break;
        }
    }

    int right_x(0);
    temp_x = x + 1;

    while (true) {
        if (tempImage.pixel(temp_x, y) != oldColor) {
            break;
        }

        tempImage.setPixel(temp_x, y, switchColor);

        if (temp_x < tempImage.width() - 1) {
            temp_x++;
            right_x = temp_x;
        } else {
            break;
        }
    }

    for (int x_(left_x + 1); x_ < right_x; ++x_) {
        if (y < 1 || y >= tempImage.height() - 1) {
            break;
        }

        if (right_x > tempImage.width()) {
            break;
        }

        QRgb currentColor = tempImage.pixel(x_, y - 1);

        if (currentColor == oldColor && currentColor != switchColor) {
            fillRecurs(x_, y - 1, switchColor, oldColor, tempImage);
        }

        currentColor = tempImage.pixel(x_, y + 1);

        if (currentColor == oldColor && currentColor != switchColor) {
            fillRecurs(x_, y + 1, switchColor, oldColor, tempImage);
        }
    }
}
