/*
    *
    * This file is a part of CorePaint.
    * A paint app for C Suite.
    * Copyright 2019 CuboCore Group
    *

    *
    * This file is taken from EasyPaint https://github.com/Gr1N/EasyPaint
    * Suitable modifications have been made to meet the needs of CorePaint.
    *

    *
    * This program is free software; you can redistribute it and/or modify
    * it under the terms of the GNU General Public License as published by
    * the Free Software Foundation; either version 3 of the License, or
    * (at your option) any later version.
    *

    *
    * This program is distributed in the hope that it will be useful,
    * but WITHOUT ANY WARRANTY; without even the implied warranty of
    * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    * GNU General Public License for more details.
    *

    *
    * You should have received a copy of the GNU General Public License
    * along with this program; if not, write to the Free Software
    * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    * MA 02110-1301, USA.
    *
*/


#include "rectangleinstrument.h"


RectangleInstrument::RectangleInstrument(QObject *parent): AbstractInstrument(parent)
{
}

void RectangleInstrument::mousePressEvent(QMouseEvent *event, ImageArea &imageArea)
{
    if (event->button() == Qt::LeftButton || event->button() == Qt::RightButton) {
        mStartPoint = mEndPoint = event->pos();
        imageArea.setIsPaint(true);
        mImageCopy = *imageArea.getImage();
        makeUndoCommand(imageArea);
    }
}

void RectangleInstrument::mouseMoveEvent(QMouseEvent *event, ImageArea &imageArea)
{
    if (imageArea.isPaint()) {
        mEndPoint = event->pos();
        imageArea.setImage(mImageCopy);

        if (event->buttons() & Qt::LeftButton) {
            paint(imageArea, false);
        } else if (event->buttons() & Qt::RightButton) {
            paint(imageArea, true);
        }
    }
}

void RectangleInstrument::mouseReleaseEvent(QMouseEvent *event, ImageArea &imageArea)
{
    if (imageArea.isPaint()) {
        imageArea.setImage(mImageCopy);

        if (event->button() == Qt::LeftButton) {
            paint(imageArea, false);
        } else if (event->button() == Qt::RightButton) {
            paint(imageArea, true);
        }

        imageArea.setIsPaint(false);
    }
}

void RectangleInstrument::paint(ImageArea &imageArea, bool isSecondaryColor, bool)
{
    QPainter painter(imageArea.getImage());
    painter.setPen(QPen(DataSingleton::Instance()->getPrimaryColor(),
                        DataSingleton::Instance()->getPenSize() * imageArea.getZoomFactor(),
                        Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin));

    if (isSecondaryColor) {
        painter.setBrush(QBrush(DataSingleton::Instance()->getSecondaryColor()));
    }

    if (mStartPoint != mEndPoint) {
        painter.drawRect(QRect(mStartPoint, mEndPoint));
    }

    imageArea.setEdited(true);
//    int rad(DataSingleton::Instance()->getPenSize() + round(sqrt((mStartPoint.x() - mEndPoint.x()) *
//                                                                 (mStartPoint.x() - mEndPoint.x()) +
//                                                                 (mStartPoint.y() - mEndPoint.y()) *
//                                                                 (mStartPoint.y() - mEndPoint.y()))));
//    mPImageArea->update(QRect(mStartPoint, mEndPoint).normalized().adjusted(-rad, -rad, +rad, +rad));
    painter.end();
    imageArea.update();
}
